@@ -7,11 +7,11 @@ import java.util.Arrays;
 import java.util.Locale;
 
 import assemble.ErrorTracker;
+import cardinality.CardinalityTracker;
 import fileIO.ByteFile;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import jgi.BBMerge;
-import jgi.LogLog;
 import shared.MetadataWriter;
 import shared.Parser;
 import shared.PreParser;
@@ -255,7 +255,7 @@ public class BloomFilterCorrectorWrapper {
 			
 //			parser.loglogk=k;
 			parser.loglogMinprob=KmerCount7MTA.minProb;
-			loglogOut=((parser.loglog&parser.loglogOut) ? new LogLog(parser) : null);
+			loglogOut=((parser.loglog&parser.loglogOut) ? CardinalityTracker.makeTracker(parser) : null);
 		}
 		
 		SamLine.SET_FROM_OK=!ecc;
@@ -891,7 +891,7 @@ public class BloomFilterCorrectorWrapper {
 	private String extout=null;
 	
 	/** For calculating kmer cardinality in output */
-	final LogLog loglogOut;
+	final CardinalityTracker loglogOut;
 	
 	/*--------------------------------------------------------------*/
 	
