@@ -805,7 +805,13 @@ public final class AminoAcid {
 		baseToComplementNumber0['T']=baseToComplementNumber0['t']=0;
 		baseToComplementNumber0['U']=baseToComplementNumber0['u']=0;
 		
-		Arrays.fill(baseToComplementExtended, (byte)-1);
+		//Invalid symbols are unchanged.
+		//This prevents crashes from -1 being out of bounds, and allows
+		//consecutive rcomp operations to restore the original sequence.
+		for(int i=0; i<baseToComplementExtended.length; i++){
+			baseToComplementExtended[i]=(byte)i;
+		}
+//		Arrays.fill(baseToComplementExtended, (byte)-1);
 		for(int i=0; i<numberToBaseExtended.length; i++){
 			char x=(char)numberToBaseExtended[i];
 			char x2=(char)numberToComplementaryBaseExtended[i];
