@@ -1754,7 +1754,7 @@ public class Data {
 	public static boolean BGZIP_VERSION_levelFlag=false; //Allows -l flag; not sure about the minimum version, but 1.9 is capable
 	private static int SH=0;
 	
-	private static int testExecute(String s, ByteBuilder bb){
+	private static int testExecute_old(String s, ByteBuilder bb){
 //		System.err.println("Testing "+s);
 		try {
 			Process p=Runtime.getRuntime().exec(s);
@@ -1794,4 +1794,61 @@ public class Data {
 		return 1;
 	}
 	
+	/** Georg Rath Version -
+	 * fixed by closing stdin and ensuring both stdout and stderr are consumed. */
+	private static int testExecute(String s, ByteBuilder bb){
+//		System.err.println("Testing "+s);
+		try {
+			ProcessBuilder pb = new ProcessBuilder(s.split("\\s+"));
+//			System.err.println("Made builder.");
+			pb.redirectErrorStream(true); //redirect to stdout
+//			System.err.println("Redirected stderr.");
+			Process p = pb.start();
+//			System.err.println("Got process.");
+			p.getOutputStream().close(); //close stdin
+//			System.err.println("Closed stdin.");
+			//			InputStream stream=p.getErrorStream();
+			InputStream stream=p.getInputStream(); //stdout
+//			System.err.println("Grabbed stdout.");
+			
+			if(bb==null){
+//				System.err.println("Reading to null.");
+				while(stream.read()>-1){}
+			}else{
+//				System.err.println("Reading to buffer.");
+				bb.clear();
+				for(int b=stream.read(); b>=0; b=stream.read()){
+					bb.append((char)b);
+					// System.err.print((char)b);
+					// System.err.print(""+(int)b);
+				}
+			}
+//			System.err.println("Finished reading.");
+			int exitValue=0;
+			while(p.isAlive()){
+				try {
+					exitValue=p.waitFor();
+				} catch (InterruptedException e) {
+					// TODO Auto-generated catch block
+					e.printStackTrace();
+				}
+			}
+//			System.err.println("Process terminated.");
+			if(exitValue>=126){
+//				System.err.println("Returning -1 due to 126.");
+				return -1;
+			}
+			// return p.exitValue()==0;
+			// System.err.println("This system has "+s+" installed.");
+		} catch (IOException e) {
+//			System.err.println("Returning -1 due to exception: "+e);
+			// System.err.println("This system does not have "+s+" installed.");
+			// TODO Auto-generated catch block
+			// e.printStackTrace();
+			return -1;
+		}
+//		System.err.println("Returning 1");
+		return 1;
+	}
+	
 }
