@@ -31,6 +31,7 @@ import stream.Read;
 import stream.SamLine;
 import stream.SamReadStreamer;
 import structures.ByteBuilder;
+import structures.EntropyTracker;
 import structures.IntList;
 import structures.ListNum;
 
@@ -86,9 +87,10 @@ public final class IceCreamFinder {
 		FASTQ.TEST_INTERLEAVED=FASTQ.FORCE_INTERLEAVED=false;
 		SamLine.SET_FROM_OK=true;
 		Shared.setBufferData(1000000);
-		Shared.FASTA_WRAP=511;
+//		Shared.FASTA_WRAP=511;
 		Data.USE_SAMBAMBA=false;//Sambamba changes PacBio headers.
 		Read.CHANGE_QUALITY=false;
+		EntropyTracker.defaultK=3;
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
@@ -213,7 +215,7 @@ public final class IceCreamFinder {
 				minLengthAfterTrimming=Integer.parseInt(b);
 			}else if(a.equals("realign")){
 				realign=Tools.parseBoolean(b);
-			}else if(a.equals("aligninverse") || a.equals("alignrc")){
+			}else if(a.equals("aligninverse") || a.equals("alignrc") || a.equals("findicecream")){
 				alignRC=Tools.parseBoolean(b);
 			}else if(a.equals("alignadapter")){
 				alignAdapter=Tools.parseBoolean(b);
@@ -283,6 +285,22 @@ public final class IceCreamFinder {
 				trimPolyA=Tools.parseBoolean(b);
 			}else if(PolymerTrimmer.parse(arg, a, b)){
 				//do nothing
+			}else if(a.equals("minentropy") || a.equals("entropy") || a.equals("entropyfilter")){
+				if(b==null || Character.isLetter(b.charAt(0))){
+					if(Tools.parseBoolean(b)){
+						entropyCutoff=0.55f;
+					}else{
+						entropyCutoff=-1;
+					}
+				}else{
+					entropyCutoff=Float.parseFloat(b);
+				}
+			}else if(a.equals("entropyblock") || a.equals("entropylength") || a.equals("entropylen") || a.equals("entlen")){
+				entropyLength=Tools.parseIntKMG(b);
+			}else if(a.equals("entropyfraction") || a.equals("entfraction")){
+				entropyFraction=Float.parseFloat(b);
+			}else if(a.equals("monomerfraction") || a.equals("maxmonomerfraction") || a.equals("mmf")){
+				maxMonomerFraction=Float.parseFloat(b);
 			}else if(a.equals("parse_flag_goes_here")){
 				long fake_variable=Tools.parseKMG(b);
 				//Set a variable here
@@ -423,6 +441,9 @@ public final class IceCreamFinder {
 		bb.appendln(Tools.numberPercent("Absent Adapter:", missingAdapterZMWs, missingAdapterZMWs*100.0/(ZMWs), 3, 8));
 		bb.appendln(Tools.numberPercent("Hidden Adapter:", hiddenAdapterZMWs, hiddenAdapterZMWs*100.0/(ZMWs), 3, 8));
 		bb.appendln(Tools.numberPercent("Ambiguous IR:", ambiguousZMWs, ambiguousZMWs*100.0/(ZMWs), 3, 8));
+//		bb.appendln(Tools.numberPercent("Low Entropy:", lowEntropyReads, lowEntropyReads*100.0/(readsProcessed), 3, 8));
+		bb.appendln(Tools.numberPercent("Low Entropy:", lowEntropyZMWs, lowEntropyZMWs*100.0/(ZMWs), 3, 8));
+		
 		bb.appendln(Tools.number("Avg Score Ratio:", iceCreamRatio/ratiosCounted, 3, 8));
 		bb.appendln(Tools.number("Score Cutoff:", minRatio2, 3, 8));
 		
@@ -486,11 +507,15 @@ public final class IceCreamFinder {
 		jo.add("Absent_Adapter_Pct", missingAdapterZMWs*100.0/(ZMWs));
 		jo.add("Hidden_Adapter", hiddenAdapterZMWs);
 		jo.add("Hidden_Adapter_Pct", hiddenAdapterZMWs*100.0/(ZMWs));
+//		jo.add("Low_Entropy", lowEntropyReads);
+//		jo.add("Low_Entropy_Pct", lowEntropyReads*100.0/(readsProcessed));
+		jo.add("Low_Entropy", lowEntropyZMWs);
+		jo.add("Low_Entropy_Pct", lowEntropyZMWs*100.0/(ZMWs));
 		jo.add("Ambiguous_Inverted_Repeat", ambiguousZMWs);
 		jo.add("Ambiguous_Inverted_Repeat_Pct", ambiguousZMWs*100.0/(ZMWs));
 		jo.add("Avg_Score_Ratio_IR", iceCreamRatio/ratiosCounted);
 		jo.add("Score_Cutoff_IR", minRatio2);
-
+		
 		jo.add("Alignment_Iterations_IR", alignmentIters);
 		jo.add("Short_Alignment_Iterations_IR", alignmentItersShort);
 		
@@ -770,6 +795,9 @@ public final class IceCreamFinder {
 			ratiosCounted+=pt.ratiosCountedT;
 			missingAdapterZMWs+=pt.missingAdapterZMWsT;
 			hiddenAdapterZMWs+=pt.hiddenAdapterZMWsT;
+			lowEntropyZMWs+=pt.lowEntropyZMWsT;
+			lowEntropyReads+=pt.lowEntropyReadsT;
+			
 			basesTrimmed+=pt.basesTrimmedT;
 			readsTrimmed+=pt.readsTrimmedT;
 			
@@ -820,8 +848,7 @@ public final class IceCreamFinder {
 	/*----------------         Inner Classes        ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	/** This class is static to prevent accidental writing to shared variables.
-	 * It is safe to remove the static modifier. */
+	/** Proecesses reads. */
 	private class ProcessThread extends Thread {
 		
 		//Constructor
@@ -835,6 +862,12 @@ public final class IceCreamFinder {
 			
 			Arrays.fill(tipBufferLeft, (byte)'N');
 			Arrays.fill(tipBufferRight, (byte)'N');
+			
+			if(entropyCutoff>=0){
+				eTracker=new EntropyTracker(false, entropyCutoff, true);
+			}else{
+				eTracker=null;
+			}
 		}
 		
 		//Called by start()
@@ -874,6 +907,24 @@ public final class IceCreamFinder {
 			return median;
 		}
 		
+		int flagLowEntropyReads(final ArrayList<Read> reads, final float minEnt, 
+				final int minLen0, final float minFract){
+			int found=0;
+			for(Read r : reads){
+				if(!r.discarded()){
+					int minLen=Tools.min((int)(r.length()*minFract), minLen0);
+					int maxBlock=eTracker.longestLowEntropyBlock(r.bases, false, maxMonomerFraction);
+					if(maxBlock>=minLen){
+						r.setDiscarded(true);
+						r.setJunk(true);
+						found++;
+//						System.err.println(r.toFasta());
+					}
+				}
+			}
+			return found;
+		}
+		
 		int flagLongReads(final ArrayList<Read> reads, int median){
 			int found=0;
 			for(Read r : reads){
@@ -886,6 +937,7 @@ public final class IceCreamFinder {
 			return found;
 		}
 		
+		/** Each list is presumed to be all reads from a ZMW, in order */
 		void processList(final ArrayList<Read> reads){
 			long numBases=0;
 			
@@ -916,7 +968,7 @@ public final class IceCreamFinder {
 						trimmed+=trimRead(r, 80);
 					}
 					
-					if(trimReads){
+					if(trimReads && adapter!=null){
 						int leftAdapterBases=alignLeftTipAdapter(r);
 						int rightAdapterBases=alignRightTipAdapter(r);
 						if(leftAdapterBases+rightAdapterBases>0){
@@ -935,6 +987,7 @@ public final class IceCreamFinder {
 						readsTrimmedT++;
 					}
 					
+					//TODO: Note again, removing intermediate reads messes up forward-rev ordering
 					if(r.length()<minLengthAfterTrimming){//Discard short trash
 						reads.set(i, null);
 						removed++;
@@ -945,6 +998,18 @@ public final class IceCreamFinder {
 				}
 			}
 			
+			if(entropyCutoff>0){
+				int bad=flagLowEntropyReads(reads, entropyCutoff, entropyLength, entropyFraction);
+				if(bad>0){
+					lowEntropyZMWsT++;
+					lowEntropyReadsT+=bad;
+					if(bad>=reads.size()){
+						if(!reads.isEmpty()){outputReads(reads, null);}
+						return; //No point in continuing...
+					}
+				}
+			}
+			
 			if(reads.isEmpty()){return;}
 
 			Read sample=null;//Read that will be searched for inverted repeat, typically median length
@@ -1079,6 +1144,8 @@ public final class IceCreamFinder {
 			if(!reads.isEmpty()){outputReads(reads, junctions);}
 		}
 		
+		//TODO: Now that I think about it.  The order of the reads is important.
+		//Since they go forward-rev-forward-rev it's imprudent to discard inner reads.
 		private void removeShortTrash(ArrayList<Read> reads) {
 			int removed=0;
 			for(int i=0; i<reads.size(); i++){
@@ -1286,6 +1353,7 @@ public final class IceCreamFinder {
 				boolean trueIceCream=(parseCustom ? ReadBuilder.isIceCream(r.id) : false);
 				trueIceCreamReads+=(trueIceCream ? 1 : 0);
 				if(r.discarded() || sendAllToDiscarded){
+//					assert(false);
 					if(bad!=null){bad.add(r);}
 					if(trueIceCream){truePositiveReadsT++;}
 					else{falsePositiveReadsT++;}
@@ -1723,6 +1791,9 @@ public final class IceCreamFinder {
 		/** Number of junctions detected in IR reads by this thread */
 		protected long junctionsOutT=0;
 		
+		protected long lowEntropyZMWsT=0;
+		protected long lowEntropyReadsT=0;
+		
 		/** True only if this thread has completed successfully */
 		boolean success=false;
 		
@@ -1745,10 +1816,12 @@ public final class IceCreamFinder {
 		final FlatAligner fla=new FlatAligner();
 //		final MultiStateAligner9PacBioAdapter msa=alignAdapter ? new MultiStateAligner9PacBioAdapter(ALIGN_ROWS, ALIGN_COLUMNS) : null;
 		/** For slowly aligning adapter sequence sectionwise */
-		final SingleStateAlignerPacBioAdapter ssa=alignAdapter ? new SingleStateAlignerPacBioAdapter(ALIGN_ROWS, ALIGN_COLUMNS, adapter.length) : null;
+		final SingleStateAlignerPacBioAdapter ssa=(alignAdapter || trimReads) ? new SingleStateAlignerPacBioAdapter(ALIGN_ROWS, ALIGN_COLUMNS, adapter.length) : null;
 		/** Alternate scoring for slow adapter alignment */
-		final MultiStateAligner9PacBioAdapter2 msa2=alignAdapter ? new MultiStateAligner9PacBioAdapter2() : null;
+		final MultiStateAligner9PacBioAdapter2 msa2=(alignAdapter || trimReads) ? new MultiStateAligner9PacBioAdapter2() : null;
 
+		final EntropyTracker eTracker;
+		
 		final long[] adapterScoresT=new long[201];
 		final long[] repeatScoresT=new long[201];
 		final byte[] tipBufferLeft=new byte[adapterTipLen+adapterTipPad];
@@ -1912,6 +1985,9 @@ public final class IceCreamFinder {
 	protected long basesTrimmed=0;
 	protected long readsTrimmed=0;
 	
+	protected long lowEntropyZMWs=0;
+	protected long lowEntropyReads=0;
+	
 	/** Total ZMWs observed */
 	protected long ZMWs=0;
 	
@@ -1953,6 +2029,20 @@ public final class IceCreamFinder {
 	
 	boolean trimPolyA=false;
 	
+	/*--------------------------------------------------------------*/
+	/*----------------        Entropy Fields        ----------------*/
+	/*--------------------------------------------------------------*/
+	
+	/** Minimum entropy to be considered "complex", on a scale of 0-1 */
+	float entropyCutoff=-1; //suggested 0.55
+	/** Minimum length of a low-entropy block to fail a read */
+	int entropyLength=350;
+	/** Minimum length of a low-entropy block as a fraction of read length;
+	 * the minimum of the two will be used */
+	float entropyFraction=0.5f;
+	
+	float maxMonomerFraction=0.74f; //Suggested...  0.74
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
 	/*--------------------------------------------------------------*/
