@@ -8,6 +8,7 @@ import java.util.BitSet;
 import java.util.HashMap;
 import java.util.Locale;
 
+import align2.QualityTools;
 import dna.ChromosomeArray;
 import dna.Data;
 import dna.Scaffold;
@@ -24,6 +25,7 @@ import shared.ReadStats;
 import shared.Shared;
 import shared.Timer;
 import shared.Tools;
+import shared.TrimRead;
 import stream.Read;
 import stream.SamLine;
 import stream.SamLineStreamer;
@@ -102,6 +104,21 @@ public class CoveragePileup {
 				outset=true;
 			}else if(a.equals("minscaf") || a.equals("covminscaf")){
 				minscaf=Integer.parseInt(b);
+			}else if(a.equals("mindepth") || a.equals("mincov")){
+				minDepthToBeCovered=Integer.parseInt(b);
+			}else if(a.equals("border")){
+				border=Integer.parseInt(b);
+			}else if(a.equals("qtrim")/* || a.equals("trim")*/){
+				if(b==null || b.length()==0){qtrimRight=qtrimLeft=true;}
+				else if(b.equalsIgnoreCase("left") || b.equalsIgnoreCase("l")){qtrimLeft=true;qtrimRight=false;}
+				else if(b.equalsIgnoreCase("right") || b.equalsIgnoreCase("r")){qtrimLeft=false;qtrimRight=true;}
+				else if(b.equalsIgnoreCase("both") || b.equalsIgnoreCase("rl") || b.equalsIgnoreCase("lr")){qtrimLeft=qtrimRight=true;}
+				else if(Tools.isDigit(b.charAt(0))){
+					trimq=Float.parseFloat(b);
+					qtrimRight=trimq>0;
+				}else{qtrimRight=qtrimLeft=Tools.parseBoolean(b);}
+			}else if(a.equals("trimq") || a.equals("trimquality")){
+				trimq=Float.parseFloat(b);
 			}else if(a.equals("ss") || a.equals("samstreamer")){
 				if(b!=null && Tools.isDigit(b.charAt(0))){
 					useStreamer=true;
@@ -230,6 +247,8 @@ public class CoveragePileup {
 			}
 			
 		}
+		trimE=(float)QualityTools.phredToProbError(trimq);
+//		assert(false) : qtrimLeft+", "+qtrimRight+", "+trimq+", "+trimE;
 		
 		ReadWrite.SAMTOOLS_IGNORE_FLAG|=ReadWrite.SAM_UNMAPPED;
 		if(!USE_SECONDARY){ReadWrite.SAMTOOLS_IGNORE_FLAG|=ReadWrite.SAM_SECONDARY;}
@@ -240,7 +259,7 @@ public class CoveragePileup {
 			SamLine.PARSE_6=false;
 			SamLine.PARSE_7=false;
 			SamLine.PARSE_8=false;
-			if(k<1){SamLine.PARSE_10=false;}
+			if(k<1 && trimq<1){SamLine.PARSE_10=false;}
 			SamLine.PARSE_OPTIONAL=false;
 		}
 		
@@ -289,6 +308,10 @@ public class CoveragePileup {
 			assert(covstats==null || covstats.indexOf('#')>=0) : "Output filenames must contain '#' symbol for strand-specific output.";
 		}
 		
+		if(!Tools.testInputFiles(false, true, in1, in2, reference)){
+			throw new RuntimeException("\nCan't read some input files.\n");  
+		}
+		
 		if(!Tools.testOutputFiles(overwrite, append, false, basecov, bincov, normcov, normcovOverall, histogram, covstats, outrpkm)){
 			throw new RuntimeException("\n\noverwrite="+overwrite+"; Can't write to output files "+
 					basecov+", "+bincov+", "+normcov+", "+normcovOverall+", "+histogram+", "+covstats+", "+outrpkm+"\n");
@@ -920,10 +943,57 @@ public class CoveragePileup {
 		return false;
 	}
 	
+	private int trim(SamLine sl){
+		assert(border>0 || (trimq>=0 && (qtrimLeft || qtrimRight)));
+		Read r=null;
+		Scaffold sc=null;
+		int leftTrimAmount=border, rightTrimAmount=border;
+		if(border>0){
+			r=sl.toRead(false);
+			sc=table.get(sl.rnameS());
+			assert(sc!=null) : sl+"\n\n"+sl.rnameS()+"\n\n"+table;
+			int skipTrimRange=Tools.max(10, border+5);
+			if(r.start<skipTrimRange){
+				if(r.strand()==Shared.PLUS){leftTrimAmount=0;}
+				else{rightTrimAmount=0;}
+			}
+			if(r.stop>sc.length-skipTrimRange){
+				if(r.strand()==Shared.PLUS){rightTrimAmount=0;}
+				else{leftTrimAmount=0;}
+			}
+		}
+		final int len0=sl.length();
+		if(qtrimLeft || qtrimRight){
+			long packed=TrimRead.testOptimal(sl.seq, sl.qual, trimE);
+			if(qtrimLeft){leftTrimAmount=Tools.max(leftTrimAmount, (int)((packed>>32)&0xFFFFFFFFL));}
+			if(qtrimRight){rightTrimAmount=Tools.max(rightTrimAmount, (int)((packed)&0xFFFFFFFFL));}
+//			assert(false) : qtrimLeft+", "+qtrimRight+", "+trimq+", "+trimE+", "+rightTrimAmount+"\n"+sl+"\n";
+		}
+		final int trimmed;
+		if(leftTrimAmount<1 && rightTrimAmount<1){trimmed=0;}
+		else{
+			if(r==null){r=sl.toRead(false);}
+			if(sc==null){sc=table.get(sl.rnameS());}
+			int scaflen=(sc==null ? 1999999999 : sc.length);
+			trimmed=TrimRead.trimReadWithMatch(r, sl, leftTrimAmount, rightTrimAmount, 0, scaflen, false);
+		}
+//		assert(trimmed==len0-sl.length()) : trimmed+", "+len0+", "+sl.length();
+//		assert(rightTrimAmount>0) : qtrimLeft+", "+qtrimRight+", "+trimq+", "+trimE+", "+rightTrimAmount+"\n"+sl+"\n";
+		return trimmed;
+	}
+	
 	public boolean processSamLine(SamLine sl){
 		readsProcessed++;
 		basesProcessed+=sl.length();
 		if(sl.duplicate() && !INCLUDE_DUPLICATES){return false;}
+		
+		if(border>0 || (trimq>=0 && (qtrimLeft || qtrimRight))){
+			if(sl.mapped() && sl.seq!=null){
+				int trimmed=trim(sl);
+				if(sl.length()<1 || trimmed<0){return false;}//trimmed<0 implies everything was trimmed
+			}
+		}
+		
 		final int kmers=Tools.countKmers(sl.seq, k);
 		final double cKmers=sl.qual==null ? kmers : Tools.countCorrectKmers(sl.qual, k);
 		kmersProcessed+=kmers;
@@ -1265,7 +1335,7 @@ public class CoveragePileup {
 						int x=ca.get(i);
 						hist.increment(Tools.min(x, histmax));
 //						sum+=x;
-						if(x>0){covered++;}
+						if(x>=minDepthToBeCovered){covered++;}
 					}
 					if(bits32){
 						int[] array=((CoverageArray3)ca).array;
@@ -1989,6 +2059,13 @@ public class CoveragePileup {
 	public int minMapq=0;
 	public boolean useStreamer=true;
 	public int streamerThreads=2;
+	public int minDepthToBeCovered=1;
+	
+	private boolean qtrimLeft=false;
+	private boolean qtrimRight=false;
+	private float trimq=-1;
+	private final float trimE;
+	private int border=0;
 	
 	/** Don't print coverage info for scaffolds shorter than this */
 	public int minscaf=0;
