@@ -9,6 +9,7 @@ import java.util.HashSet;
 import java.util.Locale;
 import java.util.Random;
 
+import cardinality.CardinalityTracker;
 import dna.AminoAcid;
 import dna.Data;
 import fileIO.ByteFile;
@@ -191,6 +192,10 @@ public class ReformatReads {
 				k=Integer.parseInt(b);
 			}
 			
+			else if(a.equalsIgnoreCase("lastUnderscoreToSpace") || a.equalsIgnoreCase("underscoreToSpace")){
+				lastUnderscoreToSpace=Tools.parseBoolean(b);
+			}
+			
 			else if(a.equals("fixheader") || a.equals("fixheaders") || a.equals("fixnames")){
 				fixHeaders=Tools.parseBoolean(b);
 			}
@@ -209,6 +214,10 @@ public class ReformatReads {
 				padQ=(byte)Integer.parseInt(b);
 			}
 			
+			else if(a.equals("minsubs")){
+				minsubs=Integer.parseInt(b);
+			}
+			
 			else if(parser.parse(arg, a, b)){
 				//do nothing
 			}else if(parser.in1==null && i==0 && !arg.contains("=") && (arg.toLowerCase().startsWith("stdin") || new File(arg).exists())){
@@ -278,7 +287,7 @@ public class ReformatReads {
 			inslenfilter=parser.inslenfilter;
 			editfilter=parser.editfilter;
 			
-			USE_EDIT_FILTER=(subfilter>-1 || delfilter>-1 || insfilter>-1 || indelfilter>-1 || dellenfilter>-1 || inslenfilter>-1 || editfilter>-1 || clipfilter>-1);
+			USE_EDIT_FILTER=(subfilter>-1 || minsubs>-1 || delfilter>-1 || insfilter>-1 || indelfilter>-1 || dellenfilter>-1 || inslenfilter>-1 || editfilter>-1 || clipfilter>-1);
 
 			setInterleaved=parser.setInterleaved;
 			
@@ -296,7 +305,7 @@ public class ReformatReads {
 			extin=parser.extin;
 			extout=parser.extout;
 			
-			loglog=(parser.loglog ? new LogLog(parser) : null);
+			loglog=(parser.loglog ? CardinalityTracker.makeTracker(parser) : null);
 			silent=Parser.silent;
 		}
 		
@@ -610,6 +619,11 @@ public class ReformatReads {
 						fixHeader(r2);
 					}
 					
+					if(lastUnderscoreToSpace){
+						underscoreToSpace(r1);
+						underscoreToSpace(r2);
+					}
+					
 					if(readstats!=null){
 						if(MAKE_QHIST){readstats.addToQualityHistogram(r1);}
 						if(MAKE_BHIST){readstats.addToBaseHistogram(r1);}
@@ -1332,6 +1346,7 @@ public class ReformatReads {
 		
 		boolean bad=false;
 		bad=bad||(subfilter>=0 && sub>subfilter);
+		bad=bad||(minsubs>=0 && sub<minsubs);
 		bad=bad||(clipfilter>=0 && clip>clipfilter);
 		bad=bad||(insfilter>=0 && inscount>insfilter);
 		bad=bad||(delfilter>=0 && delcount>delfilter);
@@ -1451,6 +1466,20 @@ public class ReformatReads {
 	
 	/*--------------------------------------------------------------*/
 	
+	public static final void underscoreToSpace(Read r){
+		if(r!=null){
+			r.id=underscoreToSpace(r.id);
+		} 
+	}
+	
+	public static String underscoreToSpace(String header){
+		int x=header.lastIndexOf('_');
+		if(x<0){return header;}
+		byte[] bytes=header.getBytes();
+		bytes[x]=' ';
+		return new String(bytes);
+	}
+	
 	public static final void fixHeader(Read r){
 		if(r!=null){
 			r.id=fixHeader(r.id);
@@ -1529,7 +1558,7 @@ public class ReformatReads {
 	/*--------------------------------------------------------------*/
 	
 	/** For calculating kmer cardinality */
-	private final LogLog loglog;
+	private final CardinalityTracker loglog;
 	
 	private int k=0;
 	
@@ -1575,6 +1604,7 @@ public class ReformatReads {
 	private float minIdFilter=-1;
 	private float maxIdFilter=-1;
 	private int subfilter=-1;
+	private int minsubs=-1;
 	private int clipfilter=-1;
 	private int delfilter=-1;
 	private int insfilter=-1;
@@ -1633,6 +1663,9 @@ public class ReformatReads {
 	/** Replace problematic symbols in read headers. */
 	private boolean fixHeaders;
 	
+	/** Change the last underscore to a space, to fix UCSF headers */
+	private boolean lastUnderscoreToSpace=false;
+	
 	private boolean useSharedHeader;
 	
 	private byte[] remap1=null, remap2=null;
