@@ -6,6 +6,9 @@ import java.util.Arrays;
 import java.util.HashSet;
 
 import align2.QualityTools;
+import cardinality.CardinalityTracker;
+import cardinality.LogLog16;
+import cardinality.LogLog2;
 import dna.AminoAcid;
 import dna.Data;
 import fileIO.ByteFile;
@@ -26,6 +29,7 @@ import stream.ReadStreamByteWriter;
 import stream.ReadStreamWriter;
 import stream.SamLine;
 import stream.SamStreamer;
+import structures.EntropyTracker;
 import structures.IntList;
 import tax.TaxTree;
 import var2.CallVariants;
@@ -117,7 +121,9 @@ public class Parser {
 				loglogOut=Tools.parseBoolean(b);
 			}
 		}else if(a.equals("buckets") || a.equals("loglogbuckets")){
-			loglogbuckets=Integer.parseInt(b);
+			loglogbuckets=Tools.parseIntKMG(b);
+			assert(loglogbuckets>0);
+			loglogbuckets=CardinalityTracker.powerOf2AtLeast(loglogbuckets);
 		}else if(a.equals("loglogbits")){
 			loglogbits=Integer.parseInt(b);
 		}else if(a.equals("loglogk") || a.equals("cardinalityk") || a.equals("kcardinality")){
@@ -132,9 +138,32 @@ public class Parser {
 			loglogseed=Long.parseLong(b);
 		}else if(a.equals("loglogminprob")){
 			loglogMinprob=Float.parseFloat(b);
+		}else if(a.equals("loglogtype")){
+			loglogType=b;
+		}else if(a.equals("loglogmean")){
+			CardinalityTracker.USE_MEAN=true;
+			CardinalityTracker.USE_MEDIAN=CardinalityTracker.USE_MWA=CardinalityTracker.USE_HMEAN=CardinalityTracker.USE_GMEAN=false;
+		}else if(a.equals("loglogmedian")){
+			CardinalityTracker.USE_MEDIAN=true;
+			CardinalityTracker.USE_MEAN=CardinalityTracker.USE_MWA=CardinalityTracker.USE_HMEAN=CardinalityTracker.USE_GMEAN=false;
+		}else if(a.equals("loglogmwa")){
+			CardinalityTracker.USE_MWA=true;
+			CardinalityTracker.USE_MEDIAN=CardinalityTracker.USE_MEAN=CardinalityTracker.USE_HMEAN=CardinalityTracker.USE_GMEAN=false;
+		}else if(a.equals("logloghmean")){
+			CardinalityTracker.USE_HMEAN=true;
+			CardinalityTracker.USE_MEDIAN=CardinalityTracker.USE_MEAN=CardinalityTracker.USE_MWA=CardinalityTracker.USE_GMEAN=false;
+		}else if(a.equals("logloggmean")){
+			CardinalityTracker.USE_GMEAN=true;
+			CardinalityTracker.USE_MEDIAN=CardinalityTracker.USE_MEAN=CardinalityTracker.USE_HMEAN=CardinalityTracker.USE_MWA=false;
+		}else if(a.equals("loglogmantissa")){
+			LogLog2.setMantissaBits(Integer.parseInt(b));
+			LogLog16.setMantissaBits(Integer.parseInt(b));
+		}else if(a.equals("loglogcounts") || a.equals("loglogcount")){
+			CardinalityTracker.trackCounts=Tools.parseBoolean(b);
 		}else{
 			return false;
 		}
+		
 		return true;
 	}
 	
@@ -697,6 +726,14 @@ public class Parser {
 			Shared.awsServers=!Tools.parseBoolean(b);
 		}
 		
+		else if(a.equals("entropyk") || a.equals("ek")){
+			EntropyTracker.defaultK=Integer.parseInt(b);
+			EntropyTracker.setDefaultK=true;
+		}else if(a.equals("entropywindow") || a.equals("ew")){
+			EntropyTracker.defaultWindowBases=Integer.parseInt(b);
+			EntropyTracker.setDefaultWindow=true;
+		}
+		
 		else{
 			return false;
 		}
@@ -822,6 +859,18 @@ public class Parser {
 			}
 		}else if(a.equals("gcchart") || a.equals("gcplot")){
 			ReadStats.GC_PLOT_X=Tools.parseBoolean(b);
+		}else if(a.equals("entropyhistogram") || a.equals("entropyhist") || a.equals("enhist") || a.equals("enthist")){
+			ReadStats.ENTROPY_HIST_FILE=(b==null || b.equalsIgnoreCase("null") || b.equalsIgnoreCase("none")) ? null : b;
+			ReadStats.COLLECT_ENTROPY_STATS=(ReadStats.ENTROPY_HIST_FILE!=null);
+			if(ReadStats.COLLECT_ENTROPY_STATS){System.err.println("Set entropy histogram output to "+ReadStats.ENTROPY_HIST_FILE);}
+		}else if(a.equals("entropybins") || a.equals("entropyhistbins") || a.equals("entbins") || a.equals("enthistbins")){
+			if("auto".equalsIgnoreCase(b)){
+				ReadStats.ENTROPY_BINS=1000;
+//				ReadStats.ENTROPY_BINS_AUTO=true;
+			}else{
+				ReadStats.ENTROPY_BINS=Integer.parseInt(b);
+//				ReadStats.ENTROPY_BINS_AUTO=false;
+			}
 		}else if(a.equals("timehistogram") || a.equals("thist")){
 			ReadStats.TIME_HIST_FILE=(b==null || b.equalsIgnoreCase("null") || b.equalsIgnoreCase("none")) ? null : b;
 			ReadStats.COLLECT_TIME_STATS=(ReadStats.TIME_HIST_FILE!=null);
@@ -1357,6 +1406,7 @@ public class Parser {
 	/*----------------        Static Fields         ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	public static String loglogType="LogLog2";
 	public static boolean silent=false;
 	
 	private static boolean setTrimRname=false;
