@@ -11,6 +11,7 @@ import fileIO.TextStreamWriter;
 import stream.Read;
 import stream.SamLine;
 import structures.ByteBuilder;
+import structures.EntropyTracker;
 import structures.LongList;
 
 
@@ -120,6 +121,14 @@ public class ReadStats {
 			gcHist=null;
 		}
 		
+		if(COLLECT_ENTROPY_STATS){
+			entropyHist=new long[ENTROPY_BINS+1];
+			eTracker=new EntropyTracker(Shared.AMINO_IN, 0, true);
+		}else{
+			entropyHist=null;
+			eTracker=null;
+		}
+		
 		if(COLLECT_ERROR_STATS){
 			errorHist=new LongList(100);
 		}else{
@@ -250,6 +259,12 @@ public class ReadStats {
 				}
 			}
 			
+			if(COLLECT_ENTROPY_STATS){
+				for(int i=0; i<rs.entropyHist.length; i++){
+					x.entropyHist[i]+=rs.entropyHist[i];
+				}
+			}
+			
 			if(COLLECT_IDENTITY_STATS){
 				for(int i=0; i<rs.idHist.length; i++){
 					x.idHist[i]+=rs.idHist[i];
@@ -443,6 +458,32 @@ public class ReadStats {
 		gcMaxReadLen=Tools.max(len, gcMaxReadLen);
 	}
 	
+	public void addToEntropyHistogram(final Read r1){
+		if(r1==null){return;}
+		final Read r2=r1.mate;
+		final int len1=r1.length(), len2=r1.mateLength();
+		
+		final float entropy1=(len1>0 ? eTracker.averageEntropy(r1.bases, false) : -1);
+		final float entropy2=(len2>0 ? eTracker.averageEntropy(r2.bases, false) : -1);
+		if(/* usePairEntropy */ false){
+			final float entropy;
+			if(r2==null){
+				entropy=entropy1;
+			}else{
+				entropy=(entropy1*len1+entropy2*len2)/(len1+len2);
+			}
+			addToEntropyHistogram(entropy, len1+len2);
+		}else{
+			addToEntropyHistogram(entropy1, len1);
+			addToEntropyHistogram(entropy2, len2);
+		}
+	}
+	
+	private void addToEntropyHistogram(final float entropy, final int len){
+		if(entropy<0 || len<1){return;}
+		entropyHist[Tools.min(ENTROPY_BINS, (int)(entropy*(ENTROPY_BINS+1)))]++;
+	}
+	
 	public void addToIdentityHistogram(final Read r){
 		if(r==null){return;}
 		addToIdentityHistogram(r, 0);
@@ -748,7 +789,7 @@ public class ReadStats {
 		return Tools.testOutputFiles(overwrite, append, allowDuplicates,
 				AVG_QUAL_HIST_FILE, QUAL_HIST_FILE, BQUAL_HIST_FILE, BQUAL_HIST_OVERALL_FILE, QUAL_COUNT_HIST_FILE,
 				MATCH_HIST_FILE, INSERT_HIST_FILE, BASE_HIST_FILE, QUAL_ACCURACY_FILE, INDEL_HIST_FILE, ERROR_HIST_FILE, LENGTH_HIST_FILE,
-				GC_HIST_FILE, IDENTITY_HIST_FILE, TIME_HIST_FILE);
+				GC_HIST_FILE, ENTROPY_HIST_FILE, IDENTITY_HIST_FILE, TIME_HIST_FILE);
 	}
 	
 	public static boolean writeAll(){
@@ -770,6 +811,7 @@ public class ReadStats {
 			if(ERROR_HIST_FILE!=null){rs.writeErrorToFile(ERROR_HIST_FILE);}
 			if(LENGTH_HIST_FILE!=null){rs.writeLengthToFile(LENGTH_HIST_FILE);}
 			if(GC_HIST_FILE!=null){rs.writeGCToFile(GC_HIST_FILE, true);}
+			if(ENTROPY_HIST_FILE!=null){rs.writeEntropyToFile(ENTROPY_HIST_FILE, true);}
 			if(IDENTITY_HIST_FILE!=null){rs.writeIdentityToFile(IDENTITY_HIST_FILE, true);}
 			if(TIME_HIST_FILE!=null){rs.writeTimeToFile(TIME_HIST_FILE);}
 			
@@ -1354,6 +1396,46 @@ public class ReadStats {
 		errorState|=bsw.errorState;
 	}
 	
+	public void writeEntropyToFile(String fname, boolean printZeros){
+		final long[] hist=entropyHist;
+		
+		final int bins=hist.length;
+		final double mult=1.0/Tools.max(1, bins-1);
+		final long total=Tools.sum(hist);
+		final long max=Tools.max(hist);
+		final double countsPerX=Tools.max(1, ((max*1000.0)/40));
+		final double fractionMult=1.0/Tools.max(1, total);
+		long sum=0;	
+		
+		double mean=Tools.averageHistogram(hist)*mult;
+		double median=Tools.percentileHistogram(hist, 0.5)*mult;
+		double mode=Tools.calcModeHistogram(hist)*mult;
+		double stdev=Tools.standardDeviationHistogram(hist)*mult;
+		
+		ByteBuilder bb=new ByteBuilder(256);
+		bb.append("#Mean\t").append(mean, 6).nl();
+		bb.append("#Median\t").append(median, 6).nl();
+		bb.append("#Mode\t").append(mode, 6).nl();
+		bb.append("#STDev\t").append(stdev, 6).nl();
+		bb.append("#Value\tCount\n");
+		
+		ByteStreamWriter bsw=new ByteStreamWriter(fname, overwrite, false, false);
+		bsw.start();
+		bsw.print(bb);
+		
+		for(int i=0; i<bins; i++){
+			long x=hist[i];
+			sum+=x;
+			if(x>0 || printZeros){
+				bb.clear().append(i*mult, 4).tab().append(x).nl();
+				bsw.print(bb);
+			}
+		}
+		bsw.poison();
+		bsw.waitForFinish();
+		errorState|=bsw.errorState;
+	}
+	
 	public void writeIdentityToFile(String fname, boolean printZeros){
 		final long[] hist, histb;
 		if(ID_BINS_AUTO && idMaxReadLen+1<idHist.length){
@@ -1420,8 +1502,10 @@ public class ReadStats {
 	public final long[] qualSub;
 	public final long[] qualIns;
 	public final long[] qualDel;
-	
+
 	public final long[] gcHist;
+	public final long[] entropyHist;
+	public final EntropyTracker eTracker;
 	public final long[] idHist;
 	public final long[] idBaseHist;
 	private int gcMaxReadLen=1;
@@ -1453,16 +1537,22 @@ public class ReadStats {
 	public static final int MAXDELLEN=1000;
 	public static final int MAXDELLEN2=1000000;
 	public static final int DEL_BIN=100;
-	public static int GC_BINS=100;
 	public static int ID_BINS=100;
 	public static boolean ID_BINS_AUTO=false;
+	public static int GC_BINS=100;
 	public static boolean GC_BINS_AUTO=false;
 	public static boolean GC_PLOT_X=false;
+	public static int ENTROPY_BINS=1000;
 
 	public static double GCMean;
 	public static double GCMedian;
 	public static double GCMode;
 	public static double GCSTDev;
+
+//	public static double entropyMean;
+//	public static double entropyMedian;
+//	public static double entropyMode;
+//	public static double entropySTDev;
 	
 	public boolean errorState=false;
 	
@@ -1478,14 +1568,16 @@ public class ReadStats {
 	public static boolean COLLECT_BASE_STATS=false;
 	public static boolean COLLECT_INDEL_STATS=false;
 	public static boolean COLLECT_GC_STATS=false;
+	public static boolean COLLECT_ENTROPY_STATS=false;
 	public static boolean COLLECT_ERROR_STATS=false;
 	public static boolean COLLECT_LENGTH_STATS=false;
 	public static boolean COLLECT_IDENTITY_STATS=false;
 	public static boolean COLLECT_TIME_STATS=false;
 	
 	public static boolean collectingStats(){
-		return COLLECT_QUALITY_STATS || COLLECT_QUALITY_ACCURACY || COLLECT_MATCH_STATS || COLLECT_INSERT_STATS || COLLECT_BASE_STATS
-				|| COLLECT_INDEL_STATS || COLLECT_GC_STATS || COLLECT_ERROR_STATS || COLLECT_LENGTH_STATS || COLLECT_IDENTITY_STATS || COLLECT_TIME_STATS;
+		return COLLECT_QUALITY_STATS || COLLECT_QUALITY_ACCURACY || COLLECT_MATCH_STATS || COLLECT_INSERT_STATS
+				|| COLLECT_BASE_STATS || COLLECT_INDEL_STATS || COLLECT_GC_STATS || COLLECT_ENTROPY_STATS
+				|| COLLECT_ERROR_STATS || COLLECT_LENGTH_STATS || COLLECT_IDENTITY_STATS || COLLECT_TIME_STATS;
 	}
 	
 	public static boolean usePairGC=true;
@@ -1503,6 +1595,7 @@ public class ReadStats {
 	public static String ERROR_HIST_FILE=null;
 	public static String LENGTH_HIST_FILE=null;
 	public static String GC_HIST_FILE=null;
+	public static String ENTROPY_HIST_FILE=null;
 	public static String IDENTITY_HIST_FILE=null;
 	public static String TIME_HIST_FILE=null;
 	
