@@ -303,7 +303,6 @@ public final class TrimRead implements Serializable {
 			System.err.println("leftTrimmed="+leftTrimAmount+", rightTrimmed="+rightTrimAmount+
 					", sum="+total+", final length="+r.length());
 		}
-		
 		return total;
 	}
 	
@@ -634,10 +633,11 @@ public final class TrimRead implements Serializable {
 	}
 
 	//TODO: This is slow
-	public static int trimReadWithMatch(final Read r, final SamLine sl, int leftTrimAmount, int rightTrimAmount, int minFinalLength, int scafLen,
-			boolean trimClip){
-		if(r.bases==null || (leftTrimAmount<1 && rightTrimAmount<1)){return 0;}
-		if(!r.containsNonM()){
+	//TODO: Note, returns a negative number if the whole read is subbosed to be trimmed
+	public static int trimReadWithMatch(final Read r, final SamLine sl, 
+			int leftTrimAmount, int rightTrimAmount, int minFinalLength, int scafLen, boolean trimClip){
+		if(r.bases==null || (leftTrimAmount<1 && rightTrimAmount<1 && !trimClip)){return 0;}
+		if(!r.containsNonM() && !trimClip){
 			return trimReadWithMatchFast(r, sl, leftTrimAmount, rightTrimAmount, minFinalLength);
 		}
 		
@@ -680,7 +680,7 @@ public final class TrimRead implements Serializable {
 		
 		if(leftTrimAmount<1 && rightTrimAmount<1){return 0;}
 		if(leftTrimAmount+rightTrimAmount>=r.length()){return -leftTrimAmount-rightTrimAmount;}
-
+		
 		final int oldPos=sl.pos;
 		
 		r.toLongMatchString(false);
