@@ -37,6 +37,19 @@ public class EntropyTracker {
 		this(k_, window_, amino_, -1, true);
 	}
 	
+	/**
+	 * Allows the use of passes() based on entropy.
+	 * @param k_ Kmer length.
+	 * @param window_ Window size in bases.
+	 * @param cutoff_ Entropy cutoff, 0 (no entropy) to 1 (max entropy).
+	 * @param highPass_ True passes entropy of at least cutoff; false fails.
+	 */
+	public EntropyTracker(boolean amino_, float cutoff_, boolean highPass_){
+		this((setDefaultK ? defaultK : amino_ ? 2 : defaultK), 
+				(setDefaultWindow ? defaultWindowBases : amino_ ? 25 : 50),
+				amino_, cutoff_, highPass_);
+	}
+	
 	/**
 	 * Allows the use of passes() based on entropy.
 	 * @param k_ Kmer length.
@@ -65,10 +78,12 @@ public class EntropyTracker {
 		
 		entropy=makeEntropyArray(windowKmers);
 		entropyMult=-1/Math.log(windowKmers);
+		baseCountMult=1f/windowBases;
+		baseCounts=new short[4];
 		counts=new short[kmerSpace];
 		countCounts=new short[windowKmers+2];
 		countCounts[0]=(short)windowKmers;
-		bases=new byte[windowBases];
+		baseRingBuffer=new byte[windowBases];
 		
 //		assert(false) : "\namino="+amino+", windowBases="+windowBases+", windowKmers="+windowKmers+", cutoff="+entropyCutoff+", bpb="+bitsPerBase+", mask="+
 //			Integer.toBinaryString(mask)+", space="+kmerSpace+", mult="+entropyMult+"\n"+"entropy="+Arrays.toString(entropy);
@@ -145,6 +160,12 @@ public class EntropyTracker {
 	/*----------------      Entropy Calculation     ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	public float calcMaxMonomerFraction(){
+		//int total=sum(baseCounts); should be window length
+		int max=Tools.max(baseCounts);
+		return baseCountMult*max;
+	}
+	
 	/**
 	 * Calculate entropy in current window.
 	 * @return Entropy in current window.
@@ -335,6 +356,73 @@ public class EntropyTracker {
 		return (float)avg;
 	}
 	
+	/**
+	 * Reports the longest block of consecutive bases in which all windows
+	 * are below the entropy cutoff and at least the (optional) monomer fraction.
+	 * @param bases
+	 * @param allowNs
+	 * @param maxMonomer
+	 * @return
+	 */
+	public int longestLowEntropyBlock(byte[] bases, boolean allowNs, float maxMonomerFraction){
+		//Reset the initial state
+		clear();
+		
+		//Position in sequence
+		int i=0;
+		
+		//Number of entropy measurements
+		int totalWindows=0;
+		
+		double sum=0;
+		int totalLowWindows=0;
+		int currentLowWindows=0;
+		int maxLowWindows=0;
+		
+		//Prefill the first window
+		for(final int lim=Tools.min(bases.length, windowBases); i<lim; i++){add(bases[i]);}
+		
+		//Calculate entropy for the first window.
+		//This allows entropy to pass if it is high enough even though the sequence is shorter than window length.
+		if(allowNs || ns==0){
+			totalWindows++;
+			double e=calcEntropy();
+			float mmf=calcMaxMonomerFraction();
+			sum+=e;
+			if(e<entropyCutoff && mmf>=maxMonomerFraction){
+				totalLowWindows++;
+				currentLowWindows++;
+				maxLowWindows=Tools.max(maxLowWindows, currentLowWindows);
+			}else{
+				currentLowWindows=0;
+			}
+		}
+		
+		//Calculate entropy for remaining windows
+		for(; i<bases.length; i++){
+			add(bases[i]);
+			if(allowNs || ns==0){
+				totalWindows++;
+				double e=calcEntropy();
+				float mmf=calcMaxMonomerFraction();
+				sum+=e;
+				if(e<entropyCutoff && mmf>maxMonomerFraction){
+					totalLowWindows++;
+					currentLowWindows++;
+					maxLowWindows=Tools.max(maxLowWindows, currentLowWindows);
+				}else{
+					currentLowWindows=0;
+				}
+			}
+		}
+		
+		//Calculate the average; not needed
+		double avg=(sum/(Tools.max(1, totalWindows)));
+		
+		int maxLowBlock=maxLowWindows<1 ? 0 : Tools.min(bases.length, maxLowWindows+windowBases-1);
+		return maxLowBlock;
+	}
+	
 	/**
 	 * Calculate entropy in the window and compare to the cutoff.
 	 * If Ns are important they should be handled externally with ns().
@@ -377,7 +465,7 @@ public class EntropyTracker {
 		//Test initial state
 		assert(!verify || verify()) : this;
 		
-		final byte oldBase=bases[pos]; //Leftmost base, about to be overwritten
+		final byte oldBase=baseRingBuffer[pos]; //Leftmost base, about to be overwritten
 		
 		if(verbose){System.err.println("\nAdding "+Character.toString((char)b)+
 				"; evicting "+Character.toString((char)oldBase)+"; counts="+Arrays.toString(counts)+"; countcounts="+Arrays.toString(countCounts)+", pos="+pos+", pos2="+pos2);}
@@ -386,8 +474,9 @@ public class EntropyTracker {
 		len++;
 		
 		{//Add a new rightmost base
-			bases[pos]=b;
+			baseRingBuffer[pos]=b;
 			final int n=symbolToNumber0[b];
+			baseCounts[n]++;
 			kmer=((kmer<<bitsPerBase)|n)&mask; //Generate new rightmost kmer using the new base
 			
 			//Update number of Ns in current window
@@ -435,7 +524,7 @@ public class EntropyTracker {
 		//At this point the state is inconsistent as it may have one too many kmers.
 		
 		if(pos2>=0){//Remove the leftmost base
-			final byte b2=(k>1 ? bases[pos2] : oldBase);//This is not the leftmost base, but the base to the right of the leftmost kmer
+			final byte b2=(k>1 ? baseRingBuffer[pos2] : oldBase);//This is not the leftmost base, but the base to the right of the leftmost kmer
 			final int n2=symbolToNumber0[b2];
 			//Generate old leftmost kmer using an internal base near the left end
 			kmer2=((kmer2<<bitsPerBase)|n2)&mask;
@@ -443,6 +532,7 @@ public class EntropyTracker {
 			if(verbose){System.err.println("B2: pos="+pos+", pos2="+pos2+"; b2="+Character.toString((char)b2)+"; kmer2="+kmer2);}
 
 			if(len>windowBases){//Remove a kmer, only if a base is leaving the window
+				baseCounts[n2]--;
 				//System.err.println("Removing "+kmer2);
 				
 				//Update number of Ns in current window
@@ -507,6 +597,7 @@ public class EntropyTracker {
 		currentEsum=0;
 		
 		//Clear mutable arrays.  Bases does not need to be cleared.
+		Arrays.fill(baseCounts, (short)0); //4 operations
 		Arrays.fill(counts, (short)0); //Time proportional to kmer space
 		
 		//Note - countCounts are only needed for medium speed mode.
@@ -529,6 +620,7 @@ public class EntropyTracker {
 	 * @return True.
 	 */
 	public boolean verifyClear(){
+		for(int c : baseCounts){assert(c==0) : this;}
 		for(int c : counts){assert(c==0) : this;}
 		for(int i=1; i<countCounts.length; i++){assert(countCounts[i]==0) : this;}
 		assert(kmer==0 && kmer2==0) : this;
@@ -572,7 +664,7 @@ public class EntropyTracker {
 		}
 		
 		//Count undefined symbols
-		for(byte b : bases){
+		for(byte b : baseRingBuffer){
 			if(!isFullyDefined(b)){nSum++;}
 		}
 		
@@ -636,9 +728,9 @@ public class EntropyTracker {
 	
 	/** Returns the ring buffer as a String in its correct order. */
 	public String basesToString(){
-		StringBuilder sb=new StringBuilder(bases.length);
-		for(int i=0; i<bases.length; i++){
-			byte b=(bases[(i+pos)%bases.length]);
+		StringBuilder sb=new StringBuilder(baseRingBuffer.length);
+		for(int i=0; i<baseRingBuffer.length; i++){
+			byte b=(baseRingBuffer[(i+pos)%baseRingBuffer.length]);
 			if(b==0){b='N';}
 			sb.append((char)b);
 		}
@@ -671,6 +763,10 @@ public class EntropyTracker {
 	/*----------------        Mutable Arrays        ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	/** Number of times each base occurs in current window.
+	 * Equivalent to counts if k=1. */
+	private final short[] baseCounts;
+	
 	/** Number of times each kmer occurs in current window.
 	 * Indexed by the kmer's numeric value.
 	 * counts[0] stores the count of the kmer AAA, if k=3. */
@@ -683,7 +779,7 @@ public class EntropyTracker {
 	
 	/** Ring buffer of bases in current window.
 	 * Not strictly necessary, but convenient. */
-	private final byte[] bases;
+	private final byte[] baseRingBuffer;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
@@ -714,6 +810,9 @@ public class EntropyTracker {
 	private final double entropyMult;
 	/** Array of precalculated constants */
 	private final double[] entropy;
+	
+	/** Precalculated constant equal to 1f/windowBases */
+	private final float baseCountMult;
 
 	/** Translation table yielding 0 if undefined */
 	private final byte[] symbolToNumber0;
@@ -745,6 +844,15 @@ public class EntropyTracker {
 	/*----------------        Static Fields         ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	/** Kmer length for entropy calculation */
+	public static int defaultK=5;
+	public static boolean setDefaultK=false;
+	/** Window length for entropy calculation */
+	public static int defaultWindowBases=50;
+	public static boolean setDefaultWindow=false;
+//	/** Minimum entropy to be considered "complex", on a scale of 0-1 */
+//	public static float defaultCutoff=-1;
+	
 	/** Entropy calculation mode */
 	public static int speed=FAST;
 	
