@@ -107,6 +107,23 @@ public final class LongList{
 		return Math.sqrt(sumdev2/size);
 	}
 	
+	public final double avgDif(final double x){
+		double sum=0;
+		for(int i=0; i<size; i++){
+			sum+=Tools.absdif(x, array[i]);
+		}
+		return sum/(Tools.max(1, size));
+	}
+	
+	public final double rmsDif(final double x){
+		double sum=0;
+		for(int i=0; i<size; i++){
+			double dif=Tools.absdif(x, array[i]);
+			sum+=dif*dif;
+		}
+		return Math.sqrt(sum/(Tools.max(1, size)));
+	}
+	
 	public final long sumLong(){
 		long sum=0;
 		for(int i=0; i<size; i++){
@@ -127,6 +144,55 @@ public final class LongList{
 		return size<1 ? 0 : sum()/size;
 	}
 	
+	//Ignores elements below 1
+	public final double harmonicMean(){
+		double sum=0;
+		int count=0;
+		for(int i=0; i<size; i++){
+			if(array[i]>0){
+				sum+=1.0/array[i];
+				count++;
+			}
+		}
+		double avg=sum/Tools.max(1, count);
+		return 1.0/avg;
+	}
+	
+	//Ignores elements below 1
+	public final double geometricMean(){
+		double sum=0;
+		int count=0;
+		for(int i=0; i<size; i++){
+			if(array[i]>0){
+				sum+=Math.log(array[i]);
+				count++;
+			}
+		}
+		double avg=sum/Tools.max(1, count);
+		return Math.exp(avg);
+	}
+	
+	/** Assumes list is sorted */
+	public final double medianWeightedAverage(){
+		if(size<1){return 0;}
+		int half=size/2;
+		long count=0;
+		double sum=0;
+		for(int i=0, j=size-1; i<half; i++, j--){
+			int mult=i+1;
+			double incr=(array[i]+array[j])*mult;
+			sum+=incr;
+			count+=2*mult;
+		}
+		if((size&1)==1){//odd length
+			int mult=half+1;
+			double incr=(array[half])*mult;
+			sum+=incr;
+			count+=2*mult;
+		}
+		return sum/count;
+	}
+	
 	/** Assumes list is sorted */
 	public final long median(){
 		if(size<1){return 0;}
@@ -134,6 +200,26 @@ public final class LongList{
 		return array[idx];
 	}
 	
+	/** Allows unsorted list */
+	public final long min(){
+		if(size<1){return 0;}
+		long x=array[0];
+		for(int i=1; i<size; i++){
+			x=Tools.min(x, array[i]);
+		}
+		return x;
+	}
+	
+	/** Allows unsorted list */
+	public final long max(){
+		if(size<1){return 0;}
+		long x=array[0];
+		for(int i=1; i<size; i++){
+			x=Tools.max(x, array[i]);
+		}
+		return x;
+	}
+	
 	/** Assumes list is sorted */
 	public final long mode(){
 		if(size<1){return 0;}
@@ -160,7 +246,7 @@ public final class LongList{
 		return best;
 	}
 	
-	public float percentile(double fraction){
+	public long percentile(double fraction){
 		if(size<1){return 0;}
 		int idx=percentileIndex(fraction);
 		return array[idx];
