@@ -32,7 +32,7 @@ public class TaxTree implements Serializable{
 	/**
 	 * 
 	 */
-	private static final long serialVersionUID = -8478466459993996423L;
+	private static final long serialVersionUID = 5894416521711540017L;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------        Initialization        ----------------*/
@@ -390,6 +390,7 @@ public class TaxTree implements Serializable{
 	}
 	
 	public synchronized void hashNames(boolean genusDotSpecies){
+		if(nameMap!=null){return;}
 		assert(nameMap==null);
 		assert(nameMapLower==null);
 		final int size=((int)Tools.mid(2, (nodes.length+(genusDotSpecies ? nodesPerLevelExtended[SPECIES_E] : 0))*1.5, Shared.MAX_ARRAY_LEN));
@@ -892,6 +893,59 @@ public class TaxTree implements Serializable{
 		return child==parent;
 	}
 	
+	public boolean isPlant(int taxID) {
+		TaxNode tn=getNode(taxID);
+		while(tn.id!=tn.pid){
+			if(tn.id==VIRIDIPLANTAE_ID){return true;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isAnimal(int taxID) {
+		TaxNode tn=getNode(taxID);
+		while(tn.id!=tn.pid){
+			if(tn.id==METAZOA_ID){return true;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isFungus(int taxID) {
+		TaxNode tn=getNode(taxID);
+		while(tn.id!=tn.pid){
+			if(tn.id==FUNGI_ID){return true;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isEukaryote(int taxID) {
+		TaxNode tn=getNode(taxID);
+		if(tn==null){
+			System.err.println("*** Warning: Can't find node "+taxID+" ***");
+			return false;
+		}
+		while(tn.id!=tn.pid){
+			if(tn.id==EUKARYOTA_ID){return true;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isProkaryote(int taxID) {
+		TaxNode tn=getNode(taxID);
+		if(tn==null){
+			System.err.println("*** Warning: Can't find node "+taxID+" ***");
+			return false;
+		}
+		while(tn.id!=tn.pid){
+			if(tn.id==BACTERIA_ID || tn.id==ARCHAEA_ID){return true;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
 	public int commonAncestor(final int a, final int b){
 		TaxNode an=getNode(a), bn=getNode(b);
 		assert(an!=null) : "Invalid taxID: "+a;
@@ -923,11 +977,33 @@ public class TaxTree implements Serializable{
 		return a;
 	}
 	
+	public static int parseHeaderStatic(String header){
+		if(header.length()<3){return -1;}
+		if(header.charAt(0)=='>'){header=header.substring(1);}
+		if(!header.startsWith("tid|")){return -1;}
+		int idx=3;
+		int idx2=header.indexOf('|', 4);
+		if(idx2<5){return -1;}
+		int id=-1;
+		try {
+			id=Tools.parseInt(header, idx+1, idx2);
+//			System.err.println("d"+", "+header.substring(idx+1, idx2));
+		} catch (Throwable e) {
+//			System.err.println("e"+", "+header.substring(idx+1, idx2));
+			//ignore
+		}
+		return id;
+	}
+	
 	public TaxNode parseNodeFromHeader(String header, boolean bestEffort){
 		if(header==null || header.length()<2){return null;}
 		if(header.charAt(0)=='>'){header=header.substring(1);}
 		TaxNode tn;
-		if(!SILVA_MODE){
+		if(SILVA_MODE){
+			tn=getNodeSilva(header, bestEffort);
+		}else if(UNITE_MODE){
+			tn=getNodeUnite(header, bestEffort);
+		}else{
 			final char delimiter=ncbiHeaderDelimiter(header);
 			if(delimiter==' '){
 				tn=getNodeNewStyle(header);
@@ -952,8 +1028,6 @@ public class TaxTree implements Serializable{
 //					System.err.println("F: "+tn);
 				}
 			}
-		}else{
-			tn=getNodeSilva(header, bestEffort);
 		}
 		return tn;
 	}
@@ -976,9 +1050,11 @@ public class TaxTree implements Serializable{
 		if(s.length()>=5 && s.startsWith("tid") && (s.charAt(3)=='|' || s.charAt(3)=='~') && Tools.isDigit(s.charAt(4))){
 			return getNodeOldStyle(s, s.charAt(3));
 		}
-		String[] split=s.split(";");
+		String[] split=Tools.semiPattern.split(s);
 		
 		int number=-1;
+//		final boolean chloroplast=(split.length>1 && split[split.length-1].equals("Chloroplast"));
+//		if(chloroplast){return null;}
 		for(int i=split.length-1; number<0 && i>=0; i--){
 			String last=split[i];
 			int paren=last.indexOf('(');
@@ -995,6 +1071,28 @@ public class TaxTree implements Serializable{
 		return null;
 	}
 	
+	TaxNode getNodeUnite(String s, boolean bestEffort){
+		if(s==null){return null;}
+		if(s.length()>=5 && s.startsWith("tid") && (s.charAt(3)=='|' || s.charAt(3)=='~') && Tools.isDigit(s.charAt(4))){
+			return getNodeOldStyle(s, s.charAt(3));
+		}
+		String[] split=Tools.pipePattern.split(s);
+
+		int number=-1;
+		String name=split[0];
+		String acc=split[1];
+		if(AccessionToTaxid.LOADED() && acc.length()>0){
+			number=AccessionToTaxid.get(acc);
+		}
+		if(number<1){
+			TaxNode tn=getNodeByName(name);
+			if(tn!=null){number=tn.id;}
+		}
+		
+		if(number>=0){return getNode(number);}
+		return null;
+	}
+	
 	private TaxNode getNodeOldStyle(final String s, char delimiter){
 		{
 			int index=s.indexOf(delimiter);
@@ -1235,6 +1333,22 @@ public class TaxTree implements Serializable{
 //		return null;
 //	}
 	
+	public int promote(final int taxID, int taxLevel){
+		TaxNode tn=null;
+		tn=(taxID<1 ? null : getNode(taxID));
+		tn=promote(tn, taxLevel);
+		return (tn==null ? taxID : tn.id);
+	}
+	
+	public TaxNode promote(TaxNode tn, int taxLevel){
+		while(tn!=null && tn.pid!=tn.id && tn.level<taxLevel){
+			TaxNode temp=getNode(tn.pid);
+			if(temp==null || temp==tn || temp.level>=TaxTree.LIFE || temp.level>taxLevel){break;}
+			tn=temp;
+		}
+		return tn;
+	}
+	
 	public int getParentID(int id){
 		assert(id<nodes.length) : id+", "+nodes.length+"\nYou have encountered a TaxID more recent than your NCBI dump."
 				+ "\nPlease redownload it and regenerate the taxtree.";
@@ -1319,13 +1433,23 @@ public class TaxTree implements Serializable{
 		if(list==null){list=getNodesByName(s, true);}
 		return list;
 	}
+	
+	private HashMap<String, ArrayList<TaxNode>> getMap(boolean lowercase){
+		HashMap<String, ArrayList<TaxNode>> map=(lowercase ? nameMapLower : nameMap);
+		if(map==null){
+			synchronized(this){hashNames(true);}
+			map=(lowercase ? nameMapLower : nameMap);
+		}
+		assert(map!=null) : "Tax names were not hashed.";
+		return map;
+	}
+	
 	private List<TaxNode> getNodesByName(String s, boolean lowercase){
 		if(s==null){return null;}
 		if(s.indexOf('_')>=0){s=s.replace('_', ' ');}
 		if(lowercase){s=s.toLowerCase();}
 //		System.err.println("Searching for "+s);
-		final HashMap<String, ArrayList<TaxNode>> map=(lowercase ? nameMapLower : nameMap);
-		assert(map!=null) : "Tax names were not hashed.";
+		final HashMap<String, ArrayList<TaxNode>> map=getMap(lowercase);
 		ArrayList<TaxNode> list=map.get(s);
 		if(list!=null){return list;}
 //		System.err.println("No matches for '"+s+"'");
@@ -1710,6 +1834,54 @@ public class TaxTree implements Serializable{
 		return level;
 	}
 	
+	public boolean isUnclassified(int tid){
+		TaxNode tn=getNode(tid);
+		while(tn!=null && tn.id!=tn.pid){
+			if(tn.isUnclassified()){return true;}
+			if(tn.pid==tn.id){break;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isEnvironmentalSample(int tid){
+		TaxNode tn=getNode(tid);
+		while(tn!=null && tn.id!=tn.pid){
+			if(tn.isEnvironmentalSample()){return true;}
+			if(tn.pid==tn.id){break;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public boolean isVirus(int tid){
+		TaxNode tn=getNode(tid);
+		while(tn!=null && tn.id!=tn.pid){
+			if(tn.id==VIRUSES_ID){return true;}
+			if(tn.pid==tn.id){break;}
+			tn=getNode(tn.pid);
+		}
+		return false;
+	}
+	
+	public long definedLevels(int tid){
+		long levels=0;
+		TaxNode tn=getNode(tid);
+		while(tn!=null && tn.id!=tn.pid){
+			levels=levels|(1L<<tn.level);
+		}
+		return levels;
+	}
+	
+	public long definedLevelsExtended(int tid){
+		long levels=0;
+		TaxNode tn=getNode(tid);
+		while(tn!=null && tn.id!=tn.pid){
+			levels=levels|(1L<<tn.levelExtended);
+		}
+		return levels;
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------            Fields            ----------------*/
 	/*--------------------------------------------------------------*/
@@ -1753,6 +1925,7 @@ public class TaxTree implements Serializable{
 	
 	public static boolean assignStrains=true;
 	public static boolean SILVA_MODE=false;
+	public static boolean UNITE_MODE=false;
 	public static boolean CRASH_IF_NO_GI_TABLE=true;
 
 	public static boolean verbose=false;
@@ -1831,6 +2004,7 @@ public class TaxTree implements Serializable{
 		"family", "order", "class", "phylum",
 		"kingdom", "superkingdom", "domain", "life"
 	};
+	public static final int numTaxaNames=taxaNames.length;
 	
 	private static final String[][] taxaNamesExtendedMatrix=new String[][] {
 		{"no rank"},
@@ -1848,6 +2022,7 @@ public class TaxTree implements Serializable{
 	};
 	
 	private static final String[] taxaNamesExtended=makeNamesExtended();
+	public static final int numTaxaNamesExtended=taxaNamesExtended.length;
 	
 	private static final String[] makeNamesExtended(){
 		ArrayList<String> list=new ArrayList<String>();
@@ -1934,7 +2109,13 @@ public class TaxTree implements Serializable{
 	
 	public static boolean IMG_HQ=false;
 	
-	public static final String defaultTaxPath="/global/projectb/sandbox/gaag/bbtools/tax/latest";
+	private static final String defaultTaxPathNersc="/global/projectb/sandbox/gaag/bbtools/tax/latest";
+	private static final String defaultTaxPathAws="/test1/tax/latest";
+	private static final String default16SFileNersc="/global/projectb/sandbox/gaag/bbtools/silva/16S_consensus_with_silva_maxns10_taxsorted.fa.gz";
+	private static final String default16SFileAws="/test1/16S_consensus_with_silva_maxns10_taxsorted.fa.gz";
+	private static final String default18SFileNersc="/global/projectb/sandbox/gaag/bbtools/silva/18S_consensus_silva_maxns10_taxsorted.fa.gz";
+	private static final String default18SFileAws="/test1/18S_consensus_silva_maxns10_taxsorted.fa.gz";
+	
 	private static final String defaultImgFile="TAX_PATH/imgDump.txt";
 	private static final String defaultTableFile="TAX_PATH/gitable.int1d.gz";
 	private static final String defaultTreeFile="TAX_PATH/tree.taxtree.gz";
@@ -1956,14 +2137,26 @@ public class TaxTree implements Serializable{
 			+ "TAX_PATH/shrunk.dead_nucl.accession2taxid.gz,"
 			+ "TAX_PATH/shrunk.pdb.accession2taxid.gz";
 
-	public static String TAX_PATH=defaultTaxPath;
+	public static final String defaultTaxPath(){
+		return (Shared.AWS && !Shared.NERSC) ? defaultTaxPathAws : defaultTaxPathNersc;
+	}
+
+	public static final String default16SFile(){
+		return (Shared.AWS && !Shared.NERSC) ? default16SFileAws : default16SFileNersc;
+	}
+
+	public static final String default18SFile(){
+		return (Shared.AWS && !Shared.NERSC) ? default18SFileAws : default18SFileNersc;
+	}
+
+	public static String TAX_PATH=defaultTaxPath();
 
 	public static final String defaultTableFile(){return defaultTableFile.replaceAll("TAX_PATH", TAX_PATH);}
 	public static final String defaultTreeFile(){return defaultTreeFile.replaceAll("TAX_PATH", TAX_PATH);}
 	public static final String defaultAccessionFile(){return defaultAccessionFile.replaceAll("TAX_PATH", TAX_PATH);}
 	public static final String defaultPatternFile(){return defaultPatternFile.replaceAll("TAX_PATH", TAX_PATH);}
 	public static final String defaultImgFile(){return defaultImgFile.replaceAll("TAX_PATH", TAX_PATH);}
-	public static final String defaultSizeFile(){return defaultSizeFile.replaceAll("TAX_PATH", TAX_PATH);} 
+	public static final String defaultSizeFile(){return defaultSizeFile.replaceAll("TAX_PATH", TAX_PATH);}
 	
 	private static PrintStream outstream=System.out;
 	
