@@ -120,7 +120,8 @@ public class CallVariants {
 		parser.qtrimRight=qtrimRight;
 		parser.trimq=trimq;
 		Shared.TRIM_READ_COMMENTS=Shared.TRIM_RNAME=true;
-
+		Read.IUPAC_TO_N=true;
+		
 		samFilter.includeUnmapped=false;
 		samFilter.includeSupplimentary=false;
 		samFilter.includeDuplicate=false;
@@ -639,7 +640,8 @@ public class CallVariants {
 			long a=initialCount, b=varMap.size(), c=varsPrefiltered, d=varsProcessed;
 			double amult=100.0/a;
 			double bmult=100.0/b;
-			long homozygousCount=ploidyArray[ploidyArray.length-1];
+			long homozygousCount=(ploidy<2 ? Tools.sum(ploidyArray) : ploidyArray[ploidyArray.length-1]);
+//			System.err.println(Arrays.toString(ploidyArray));
 			double homozygousRate=homozygousCount*1.0/Tools.sum(ploidyArray);
 //			outstream.println();
 			if(prefilter){
@@ -1104,6 +1106,17 @@ public class CallVariants {
 //			System.err.println("C: "+sl);
 			
 			int leftTrimAmount=border, rightTrimAmount=border;
+			if(border>0){
+				int skipTrimRange=Tools.max(10, border+5);
+				if(r.start<skipTrimRange){
+					if(r.strand()==Shared.PLUS){leftTrimAmount=0;}
+					else{rightTrimAmount=0;}
+				}
+				if(r.stop>scaf.length-skipTrimRange){
+					if(r.strand()==Shared.PLUS){rightTrimAmount=0;}
+					else{leftTrimAmount=0;}
+				}
+			}
 			if(qtrimLeft || qtrimRight){
 				long packed=TrimRead.testOptimal(r.bases, r.quality, trimE);
 				if(qtrimLeft){leftTrimAmount=Tools.max(leftTrimAmount, (int)((packed>>32)&0xFFFFFFFFL));}
