@@ -256,7 +256,7 @@ public class FilterSam {
 			//Select output buffer size based on whether it needs to be ordered
 			final int buff=(ordered ? Tools.mid(16, 128, (Shared.threads()*2)/3) : 8);
 			
-			ros=ConcurrentReadOutputStream.getStream(ffoutGood, null, null, null, buff, null, false);
+			ros=ConcurrentReadOutputStream.getStream(ffoutGood, null, null, null, buff, null, true);
 			ros.start(); //Start the stream
 		}else{ros=null;}
 		if(ffoutBad!=null){
