@@ -3,7 +3,7 @@
 usage(){
 echo "
 Written by Brian Bushnell
-Last modified January 28, 2020
+Last modified May 4, 2020
 
 Description:  Finds PacBio reads containing inverted repeats.
 These are candidate triangle reads (ice cream cones).
@@ -59,6 +59,7 @@ trimpolya=f     Trim terminal poly-A and poly-T sequences, for some isoseq
 minpolymer=5    Don't trim poly-A sequence shorter than this.
 polyerror=0.2   Max error rate for trimming poly-A.
 
+
 Speed and sensitivity:
 jni=f           Enable C code for higher speed and identical results.
 minratio=       Fraction of maximal alignment score to consider as matching.
@@ -72,6 +73,18 @@ adapterratio=0.18   Initial adapter detection sensitivity; affects speed.
 adapterratio2=.325  Final adapter detection sensitivity.
 minscore=-800   Exit alignment early if score drops below this.
 
+Entropy parameters (recommended setting is 'entropy=t'):
+minentropy=-1   Set to 0.4 or above to remove low-entropy reads;
+                range is 0-1, recommended value is 0.55.  0.7 is too high.
+                Negative numbers disable this function.
+entropyk=3      Kmer length for entropy calculation.
+entropylen=450  Reads with entropy below cutoff for at least this many
+                consecutive bases will be removed.
+entropyfraction=0.5     Alternative minimum length for short reads; the shorter
+                        of entropylen and entfraction*readlength will be used.
+entropywindow=50        Window size used for entropy calculation.
+maxmonomerfraction=0.75 (mmf) Also require this fraction of bases in each
+                        window to be the same base.
 
 Java Parameters:
 -Xmx            This will set Java's memory usage, overriding autodetection.
